/* array01 - simple tests for MathArray from libMathArray

   Copyright (C) 1995, Adam Fedor
   		 2013, Sebastian Reitenbach

*/

#ifdef NEXT_FOUNDATION
#include <foundation/NSValue.h>
#include <foundation/NSAutoreleasePool.h>
#define  reason exceptionReason
#else
#include <Foundation/NSValue.h>
#include <Foundation/NSAutoreleasePool.h>
#endif
#import "ObjectTesting.h"
#include <stdio.h>
#include <math.h>
#include "MathArray/MathArray.h"
#include "MathArray/MaskedException.h"
#include "MathArray/MAValue.h"

/* Replace value by the exponent of it's index */
id <NSNumber,ComplexNumber>
exp_array(id <NSNumber,ComplexNumber>number, unsigned *index, void *info)
{
  return [MANumber numberWithDouble:index[0]*index[1]];
}

int
main(int argc, char *argv[])
{
  NSAutoreleasePool* pool;
  MathArray *array;
  MANumber  *number;
  complex_double cd;
  unsigned  index[2];
  NSString *tmpValue, *tmpValue2;

  pool = [NSAutoreleasePool new];
  [MaskedException setMaskForAllExceptions:AbortException];

  //printf("Create 100 point vector...\n");
  array = [MathArray maWithVector:100 objCType:@encode(int)];
  PASS(array != nil, "Create 100 point vector...");

  [array maAdd:[NSNumber numberWithDouble:3.01]];
  tmpValue = [NSString stringWithFormat:@"%.1f", [[array maTotal] floatValue]];
  PASS([@"301.0" isEqual:tmpValue], "add 3.01, and compute total");

  [array maDivide:[NSNumber numberWithInt:2]];
  tmpValue = [NSString stringWithFormat:@"%.1f", [[array maTotal] floatValue]];
  PASS([@"150.5" isEqual:tmpValue], "now divided with 2");

  [array maCos];
  tmpValue = [NSString stringWithFormat:@"%.6f", [[array maTotal] floatValue]];
  PASS([@"6.574886" isEqual:tmpValue], "now take cosine");

  number = [MANumber numberWithComplexDouble:((complex_double){2, 1})];
  [array maMultiply:number];
  cd = [[array maMinimumValue] complexDoubleValue];
  tmpValue = [NSString stringWithFormat:@"%.6f", cd.real];
  tmpValue2 = [NSString stringWithFormat:@"%.7f", cd.imag];
  PASS([@"0.131498" isEqual:tmpValue] && [@"0.0657489" isEqual:tmpValue2],
	"Mult 2+i calculate minimum");

  /* Matrices */
  array = [MathArray maMatrixWithCols:10 rows:10 objCType:NULL];
  PASS(array != nil, "Create 10x10 matrix...");

  [array maPerformFunction:exp_array userInfo:NULL];
  PASS(array != nil, "perform user function");

  index[0] = 3;
  index[1] = 3;
  tmpValue = [NSString stringWithFormat:@"%g", [[array arrayValueAtIndex:index] doubleValue]];
  PASS([@"9" isEqual:tmpValue], "calculate value at (3,3)");

  tmpValue = [NSString stringWithFormat:@"%g", [[array maTotal] doubleValue]];
  PASS([@"2025" isEqual:tmpValue], "calculating the total of the matrix");

  /* Try an illegal operation */
  PASS_EXCEPTION([array maDivide: [NSNumber numberWithFloat:0.0]],
	MAFloatingPointException, "caught exception trying illegal operation");
  [pool release];
  return 0;
}
