/* array02 - tests for MathArray from libMathArray

   Copyright (C) 1995, Adam Fedor

*/

#ifdef NEXT_FOUNDATION
#include <foundation/NSValue.h>
#include <foundation/NSAutoreleasePool.h>
#else
#include <Foundation/NSValue.h>
#include <Foundation/NSAutoreleasePool.h>
#endif
#import "ObjectTesting.h"
#include <stdio.h>
#include <math.h>
#include "MathArray/MathArray.h"
#include "MathArray/MaskedException.h"
#include "MathArray/MAValue.h"
#include "MathArray/MAValueData.h"

/* Replace value by the exponent of it's index */
id <NSNumber,ComplexNumber>
exp_array(id <NSNumber,ComplexNumber>number, unsigned *index, void *info)
{
    return [MANumber numberWithDouble:index[0]*index[1]];
}

int
main()
{
  MathArray *array;
  int  index[2];
  double *data;
  NSAutoreleasePool* pool = [NSAutoreleasePool new];
  NSString *tmpValue;

  [MaskedException setMaskForAllExceptions:AbortException];

  /* Matrices */
  array = [MathArray maMatrixWithCols:64 rows:64 objCType:NULL];
  PASS(array != nil, "create 64x64 matrix...");

  printf("Perform user function...\n");
  [array maPerformFunction:exp_array userInfo:NULL];
  PASS(array != nil, "perform user function");

  printf("Perform fft with origin in center...\n");
  index[0] = 32;
  index[1] = 32;
  data = [[array mathData] mutableBytes];
  [array maShift:index];
  [array maFFt:-1];
  [array maShift:index];
  data = [[array mathData] mutableBytes];
  PASS(data != NULL, "perform fft with origin in center");

  index[0] = 32;
  index[1] = 32;
  tmpValue = [NSString stringWithFormat:@"%g", [[array arrayValueAtIndex:index] doubleValue]];
  PASS([@"992.25" isEqual:tmpValue], "test value at (32,32)" );

  tmpValue = [NSString stringWithFormat:@"%g", [[array maTotal] doubleValue]];
  PASS([@"1024" isEqual:tmpValue], "calculate total of matrix" );
  [pool release];
  return 0;
}
