/* array01 - simple tests for MathArray from libMathArray

   Copyright (C) 1995, Adam Fedor

*/

#ifdef NEXT_FOUNDATION
#import <foundation/NSValue.h>
#import <foundation/NSRange.h>
#include <foundation/NSAutoreleasePool.h>
#else
#include <Foundation/NSValue.h>
#include <Foundation/NSRange.h>
#include <Foundation/NSAutoreleasePool.h>
#endif
#import "ObjectTesting.h"
#include <stdio.h>
#include <math.h>
#include "MathArray/MathArray.h"
#include "MathArray/MaskedException.h"
#include "MathArray/MAValue.h"
#include "MathArray/MAValueData.h"

id <NSNumber,ComplexNumber>
array_func(id <NSNumber,ComplexNumber>number, unsigned *index, void *info)
{
    return [MANumber numberWithInt:index[0]];
}

int
main()
{
  MathArray *array;
  MathArray *subarray;
  NSRange range;
  unsigned int  index[2];
  int i;
  NSAutoreleasePool* pool = [NSAutoreleasePool new];
  NSString *tmpValue, *tmpValue2;
  NSArray *testArray;

  [MaskedException setMaskForAllExceptions:AbortException];

  array = [MathArray maWithVector:10 objCType:@encode(int)];
  PASS(array != nil, "create 10 point vector...");

  [array maPerformFunction:array_func userInfo:NULL];
  PASS(array != nil, "perform user function...");

  range.location = 3;
  range.length = 4;
  subarray = [array arraySubrange:&range];
  PASS(subarray != nil, "extracting subarray");

  tmpValue = [NSString stringWithFormat:@"%d", [[subarray maTotal] intValue]];
  PASS([@"18" isEqual:tmpValue], "calculating subarray total");

  index[0] = 5;
  [array setArray:subarray atIndex:index];
  testArray = [NSArray arrayWithObjects: @"0", @"1", @"2", @"3", @"4", @"3", @"4", @"5", @"6", @"9", nil];
  for (i=0; i < [[array mathData] count]; i++) {
      tmpValue = [NSString stringWithFormat:@"%d", [(NSNumber *)[[array mathData] valueAtIndex:i] intValue]];
      tmpValue2 = [NSString stringWithFormat:@"%d", [[testArray objectAtIndex:i] intValue]];
      PASS([tmpValue2 isEqual:tmpValue], "insert subarray");
  }

  array = [MathArray maMatrixWithCols:10 rows:20 objCType:NULL];
  PASS(array != nil, "create 20x10 matrix");

  [array maPerformFunction:array_func userInfo:NULL];
  PASS(array != nil, "perform user function");

  index[0] = 5;
  index[1] = 5;
  [array setArray:subarray atIndex:index];
  PASS(array != nil, "insert subarray");

  tmpValue = [NSString stringWithFormat:@"%d", [[array maTotal] intValue]];
  PASS([@"1898" isEqual:tmpValue], "calculating total of array");

  [array maMatrixMultiply:[MA_TEMPORARY(array) maTranspose]];
  PASS(array != nil, "calculate square array by matrix multiplication");

  tmpValue = [NSString stringWithFormat:@"%d", [[array maTotal] intValue]];
  PASS([@"360246" isEqual:tmpValue], "calculating total of array");

  [[array maGreater:[NSNumber numberWithInt:2470]] maWhere];
  PASS(array != nil, "where array is greater than 2470");

  tmpValue = [NSString stringWithFormat:@"%u", [array sizes][0]];
  tmpValue2 = [NSString stringWithFormat:@"%d", [[array maTotal] intValue]];
  PASS([@"26" isEqual:tmpValue] && [@"9324" isEqual:tmpValue2], 
	"calculating size and total of array")

  [array maRandomNormalMean: 0 deviation: 4];
  PASS(array != nil, "creating random array");
  //for (i=0; i < [[array mathData] count]; i++) 
  //    printf("%d ", [(NSNumber *)[[array mathData] valueAtIndex:i] intValue]);

  [pool release];
  return 0;
}
