/* array04 - simple tests for NSDecimal arrays 

   Copyright (C) 1995, Adam Fedor

*/

#define HAVE_FOUNDATION_NSDECIMAL_H

#ifdef NEXT_FOUNDATION
#include <foundation/NSValue.h>
#include <foundation/NSAutoreleasePool.h>
#define  reason exceptionReason
#else
#include <Foundation/NSValue.h>
#include <Foundation/NSAutoreleasePool.h>
#endif
#ifdef HAVE_FOUNDATION_NSDECIMAL_H
#include <Foundation/NSDecimal.h>
#include <Foundation/NSDecimalNumber.h>
#endif
#import "ObjectTesting.h"
#include <stdio.h>
#include <math.h>
#include "MathArray/MathArray.h"
#include "MathArray/MaskedException.h"
#include "MathArray/MAValue.h"

/* Replace value by the exponent of it's index */
id <NSNumber,ComplexNumber>
exp_array(id <NSNumber,ComplexNumber>number, unsigned *index, void *info)
{
  return [MANumber numberWithDouble:index[0]*index[1]];
}

int
main()
{
  NSAutoreleasePool* pool;
  MathArray *array;
  unsigned  index[2];
  NSString *tmpValue; 

  pool = [NSAutoreleasePool new];
  [MaskedException setMaskForAllExceptions:AbortException];

#ifdef HAVE_FOUNDATION_NSDECIMAL_H
  array = [MathArray maWithVector:100 objCType:@encode(NSDecimal)];
  PASS(array != nil, "create 100 point vector");

  [array maAdd:[NSNumber numberWithFloat:3.01]];
  tmpValue = [NSString stringWithFormat:@"%.1f", [[array maTotal] floatValue]];
  PASS([@"301.0" isEqual:tmpValue], "add 3.01, calculating total of array");

  [array maDivide:[NSNumber numberWithInt:2]];
  tmpValue = [NSString stringWithFormat:@"%.1f", [[array maTotal] floatValue]];
  PASS([@"150.5" isEqual:tmpValue], "divide by 2, calculating total of array");

/*
  [array maCos];
  tmpValue = [NSString stringWithFormat:@"%.6f", [[array maTotal] floatValue]];
  PASS([@"6.574887" isEqual:tmpValue], "take cosinus, calculate total of array");
*/

  /* Matrices */
  array = [MathArray maMatrixWithCols:10 rows:10 objCType: @encode(NSDecimal)];
  PASS(array != nil, "create 10x10 matrix...");

  [array maPerformFunction:exp_array userInfo:NULL];
  PASS(array != nil, "perform user function...");
  index[0] = 3;
  index[1] = 3;
  tmpValue = [NSString stringWithFormat:@"%g", [[array arrayValueAtIndex:index] doubleValue]];
  PASS([@"9" isEqual:tmpValue], "get value at (3, 3)");

  tmpValue = [NSString stringWithFormat:@"%g", [[array maTotal] doubleValue]];
  PASS([@"2025" isEqual:tmpValue], "calculating total of matrix");

  /* Try an illegal operation */
  PASS_EXCEPTION([array maDivide: [NSNumber numberWithFloat:0.0]],
	MAFloatingPointException, "caught exception trying illegal operation");

  printf("Try very small numbers...\n");
  printf("Start with zeroed 9x9 array\n");
  array = [MathArray maMatrixWithCols: 9 rows: 9 objCType: @encode(NSDecimal)];
  PASS(array != nil, "try very small numbers, start with zeroed 9x9 array");
  [array maAdd: [NSDecimalNumber decimalNumberWithString: 
		  @"3.456789012345678e-45"]];
  printf("Add 3.456789102345678e-45, Total is %s\n", 
	[[[array maTotal] stringValue] cString]);

  [array maDivide: [NSDecimalNumber decimalNumberWithString: @"8.1"]];
  printf("Divide by 8.1,             Total is %s (should by 3.456...e-44)\n", 
	[[[array maTotal] stringValue] cString]);
  tmpValue = [[array maTotal] stringValue];
  tmpValue = [tmpValue substringToIndex: ([tmpValue length] > 60)?60:[tmpValue length]];
  PASS([@"0.0000000000000000000000000000000000000000000345678901234567" isEqual:tmpValue], "divide by 8.1, calculating total of matrix");

#endif

  [pool release];
  return 0;
}
